'use strict';

var ParserHelpers = require('webpack/lib/ParserHelpers');
var OpenUI5LazyInstanceDependency = require('./OpenUI5LazyInstanceDependency');
var OpenUI5ViewDependency = require('./OpenUI5ViewDependency');
var OpenUI5RequireItemDependency = require('./OpenUI5RequireItemDependency');
var OpenUI5RequireContextDependency = require('./OpenUI5RequireContextDependency');
var ContextDependencyHelpers = require('./ContextDependencyHelpers');

class OpenUI5RequireDependencyParserPlugin {
  constructor(options) {
    this.options = options;
  }

  apply(parser) {
    var options = this.options || {};
    var globals = {
      'sap.ui.core.theming.Parameters': 'sap/ui/core/theming/Parameters',
      'sap.ui.core.LabelEnablement': 'sap/ui/core/LabelEnablement',
      'sap.ui.core.HTML': 'sap/ui/core/HTML',
      'sap.m.CheckBox': 'sap/m/CheckBox',
      'sap.viz.ui5.theming.Util': 'sap/viz/ui5/theming/Util',
      'sap.ui.unified.DateRange': 'sap/ui/unified/DateRange'
    };

    var processItem = function processItem(expr, param) {
      if (param.isString()) {
        var item = param.string;
        if (item.substr(0, 10) !== 'jquery.sap' && item.substr(0, 2) !== './') {
          item = item.replace(/\./g, '/');
        }
        var dep = new OpenUI5RequireItemDependency(item, expr.range);
        dep.loc = expr.loc;
        dep.optional = !!parser.scope.inTry;
        parser.state.current.addDependency(dep);
        return true;
      }
      return false;
    };

    var processContext = function processContext(expr, param) {
      var dep = ContextDependencyHelpers.create(OpenUI5RequireContextDependency, expr.range, param, expr, options);
      if (!dep) return false;
      dep.loc = expr.loc;
      dep.optional = !!parser.scope.inTry;
      parser.state.current.addDependency(dep);
      return true;
    };

    var processArray = function processArray(expr, param) {
      param.items.forEach(function (param) {
        processItem(expr, param);
      });
      return true;
    };

    var processGlobal = function processGlobal(expr, request) {
      var dep = new OpenUI5RequireItemDependency(request, expr.range, true);
      dep.loc = expr.loc;
      dep.optional = !!parser.scope.inTry;
      parser.state.current.addDependency(dep);
      return true;
    };

    var processRemove = ParserHelpers.toConstantDependency(parser, process.env.NODE_ENV === 'production' ? '' : 'console.warn("UI5 tried to dynamically require a module. If it doesn\'t exist in the bundle a error might follow.")');

    parser.hooks.call.for('sap.ui.require').tap('OpenUI5Plugin', function (expr) {
      var param = parser.evaluateExpression(expr.arguments[0]);
      if (expr.arguments[0].type === 'Literal') {
        processItem(expr, param);
        return true;
      } else if (expr.arguments[0].type === 'ArrayExpression') {
        processArray(expr, param);
        return true;
      }
      return false;
    });

    parser.hooks.call.for('sap.ui.requireSync').tap('OpenUI5Plugin', function (expr) {
      var param = parser.evaluateExpression(expr.arguments[0]);

      var result = processItem(expr, param);
      if (!result) {
        result = processContext(expr, param);
      }
      return result;
    });

    parser.hooks.call.for('lazyInstanceof').tap('OpenUI5Plugin', function (expr) {
      var param1 = parser.evaluateExpression(expr.arguments[1]);
      if (param1.string !== 'sap/ui/app/Application') {
        var dep = new OpenUI5LazyInstanceDependency(param1.string, expr.arguments[0].name, expr.range);
        dep.loc = expr.loc;
        dep.optional = !!parser.scope.inTry;
        parser.state.current.addDependency(dep);
        return true;
      }
      return false;
    });

    parser.hooks.call.for('createView').tap('OpenUI5Plugin', function (expr) {
      var param0 = parser.evaluateExpression(expr.arguments[0]);
      var dep = new OpenUI5ViewDependency(param0.string, expr.arguments[1].name, expr.range);
      dep.loc = expr.loc;
      dep.optional = !!parser.scope.inTry;
      parser.state.current.addDependency(dep);
      return true;
    });

    parser.hooks.call.for('jQuery.sap.require').tap('OpenUI5Plugin', function (expr) {
      var param = parser.evaluateExpression(expr.arguments[0]);
      if (expr.arguments[0].type === 'Literal') {
        processItem(expr, param);
        return true;
      }
      processRemove(expr, param);
      return false;
    });

    parser.hooks.call.for('$.sap.require').tap('OpenUI5Plugin', function (expr) {
      var param = parser.evaluateExpression(expr.arguments[0]);
      if (expr.arguments[0].type === 'Literal') {
        processItem(expr, param);
        return true;
      }
      return false;
    });

    Object.keys(globals).forEach(function (key) {
      parser.hooks.expression.for(key).tap('OpenUI5Plugin', function (expr) {
        return processGlobal(expr, globals[key]);
      });
    });
  }
}
module.exports = OpenUI5RequireDependencyParserPlugin;